{% if with_docs %}

[testenv:docs]
basepython = python3
skip_install = false
{% if docs_deps %}
deps =
  {% for line in docs_deps %}
    %(line)s
  {% endfor %}
{% endif %}
{% if not with_sphinx_doctests %}
extras =
    docs
{% endif %}
commands_pre =
commands =
    sphinx-build -b html -d docs/_build/doctrees docs docs/_build/html
{% if with_sphinx_doctests %}
    sphinx-build -b doctest -d docs/_build/doctrees docs docs/_build/doctest
{% endif %}
{% endif %}
