[tox]
minversion = 3.18
envlist =
    release-check
    lint
{% for py_version in supported_python_versions %}
    py%(py_version)s
{% endfor %}
{% if with_future_python %}
    py%(future_python_shortversion)s
{% endif %}
{% if with_pypy %}
    pypy3
{% endif %}
{% if with_docs %}
    docs
{% endif %}
{% if with_coverage %}
    coverage
{% endif %}
{% for line in additional_envlist %}
    %(line)s
{% endfor %}
