
{% include 'tox-release-check.j2' %}

[testenv:lint]
description = This env runs all linters configured in .pre-commit-config.yaml
basepython = python3
skip_install = true
deps =
    pre-commit
commands_pre =
commands =
    pre-commit run --all-files --show-diff-on-failure
