{% if zest_releaser_options %}

[zest.releaser]
{% for line in zest_releaser_options %}
%(line)s
{% endfor %}
{% endif %}

[flake8]
doctests = 1
{% if config_type == 'buildout-recipe' %}
# provided to doctests by buildoutSetUp()
builtins = write, system, cat, join
{% endif %}
{% if config_type == 'zope-product' %}
no-accept-encodings = True
htmldir = parts/flake8
{% endif %}
{% for line in flake8_additional_config %}
%(line)s
{% endfor %}

[check-manifest]
ignore =
    .editorconfig
    .meta.toml
{% if with_docs %}
    docs/_build/html/_sources/*
{% endif %}
{% if with_sphinx_doctests %}
    docs/_build/doctest/*
{% endif %}
{% for line in check_manifest_additional_ignores %}
    %(line)s
{% endfor %}
{% if check_manifest_ignore_bad_ideas %}
ignore-bad-ideas =
  {% for line in check_manifest_ignore_bad_ideas %}
    %(line)s
  {% endfor %}
{% endif %}

[isort]
force_single_line = True
combine_as_imports = True
sections = FUTURE,STDLIB,THIRDPARTY,ZOPE,FIRSTPARTY,LOCALFOLDER
known_third_party =%(isort_known_third_party)s
known_zope =%(isort_known_zope)s
known_first_party =%(isort_known_first_party)s
{% if isort_known_local_folder %}
known_local_folder =%(isort_known_local_folder)s
{% endif %}
default_section = ZOPE
line_length = 79
lines_after_imports = 2
{% for line in isort_additional_config %}
%(line)s
{% endfor %}
