
[testenv]
{% if config_type != 'c-code' %}
usedevelop = true
{% endif %}
{% if config_type == "pure-python" %}
package = wheel
wheel_build_env = .pkg
{% endif %}
{% if with_future_python %}
pip_pre = py%(future_python_shortversion)s: true
{% endif %}
deps =
    setuptools %(setuptools_version_spec)s
  {% for line in testenv_deps %}
    %(line)s
  {% endfor %}
  {% if with_sphinx_doctests and with_future_python %}
    Sphinx
  {% endif %}
{% if testenv_setenv %}
setenv =
  {% for line in testenv_setenv %}
    %(line)s
  {% endfor %}
{% endif %}
{% if testenv_commands_pre %}
commands_pre =
  {% for line in testenv_commands_pre %}
    %(line)s
  {% endfor %}
{% endif %}
commands =
{% if testenv_commands %}
  {% for line in testenv_commands %}
    %(line)s
  {% endfor %}
{% else %}
    zope-testrunner --test-path=src {posargs:-vc}
{% endif %}
{% if with_sphinx_doctests %}
    sphinx-build -b doctest -d {envdir}/.cache/doctrees docs {envdir}/.cache/doctest
{% endif %}
extras =
    test
  {% if with_sphinx_doctests %}
    docs
  {% endif %}
  {% for line in testenv_additional_extras %}
    %(line)s
  {% endfor %}
{% for line in testenv_additional %}
%(line)s
{% endfor %}

[testenv:setuptools-latest]
basepython = python3
deps =
    git+https://github.com/pypa/setuptools.git\#egg=setuptools
{% for line in testenv_deps %}
    %(line)s
{% endfor %}
