[testenv:release-check]
description = ensure that the distribution is ready to release
basepython = python3
skip_install = true
deps =
{% for build_requirement in build_requirements %}
    %(build_requirement)s
{% endfor %}
    twine
    build
    check-manifest
    check-python-versions >= 0.20.0
    wheel
commands_pre =
commands =
    check-manifest
    check-python-versions --only pyproject.toml,setup.py,tox.ini,.github/workflows/tests.yml
    python -m build --sdist --no-isolation
    twine check dist/*
