minimum_pre_commit_version: '3.6'
repos:
  - repo: https://github.com/pycqa/isort
    rev: "7.0.0"
    hooks:
    - id: isort
  - repo: https://github.com/hhatto/autopep8
    rev: "v2.3.2"
    hooks:
    - id: autopep8
      args: [--in-place, --aggressive, --aggressive]
  - repo: https://github.com/asottile/pyupgrade
    rev: v3.21.0
    hooks:
    - id: pyupgrade
      args: [--py%(oldest_python_version)s-plus]
{% if pyupgrade_exclude %}
      exclude: %(pyupgrade_exclude)s
{% endif %}
  - repo: https://github.com/isidentical/teyit
    rev: 0.4.3
    hooks:
    - id: teyit
{% if teyit_exclude %}
      exclude: %(teyit_exclude)s
{% endif %}
  - repo: https://github.com/PyCQA/flake8
    rev: "7.3.0"
    hooks:
    - id: flake8
      additional_dependencies:
        - flake8-debugger == 4.1.2
