[build-system]
requires = [
    "setuptools",
    "wheel",
]
build-backend = "setuptools.build_meta"

[tool.coverage.run]
branch = true
source = ["%(coverage_run_source)s"]
relative_files = true

[tool.coverage.report]
fail_under = %(coverage_fail_under)s
precision = 2
ignore_errors = true
show_missing = true
exclude_lines = [
    "pragma: no cover",
    "pragma: nocover",
    "except ImportError:",
    "raise NotImplementedError",
    "if __name__ == '__main__':",
    "self.fail",
    "raise AssertionError",
    "raise unittest.Skip",
]

[tool.coverage.html]
directory = "parts/htmlcov"

[tool.coverage.paths]
source = [
    "src/",
    ".tox/*/lib/python*/site-packages/",
    ".tox/pypy*/site-packages/",
]
