[run]
source = %(coverage_run_source)s
branch = true
parallel = true
{% for line in coverage_run_additional_config %}
%(line)s
{% endfor %}

[paths]
source =
    src/
    .tox/*/lib/python*/site-packages/
    .tox/pypy*/site-packages/

[report]
precision = 2
ignore_errors = True
exclude_lines =
    except ImportError:
    if __name__ == '__main__':
    pragma: no cover
    pragma: nocover
    raise AssertionError
    raise NotImplementedError
    raise unittest.Skip
    self.fail\(

[html]
directory = parts/htmlcov
