{% set with_coverage = False %}
{% include 'tox-envlist.j2' %}

[testenv]
skip_install = true
deps =
    setuptools %(setuptools_version_spec)s
    zc.buildout
    wheel
{% for line in testenv_deps %}
    %(line)s
{% endfor %}
setenv =
{% if testenv_setenv %}
  {% for line in testenv_setenv %}
    %(line)s
  {% endfor %}
{% endif %}
commands_pre =
{% if testenv_commands_pre %}
  {% for line in testenv_commands_pre %}
    %(line)s
  {% endfor %}
{% else %}
    {envbindir}/buildout -nc {toxinidir}/buildout.cfg buildout:directory={envdir}
{% endif %}
commands =
{% if testenv_commands %}
  {% for line in testenv_commands %}
    %(line)s
  {% endfor %}
{% else %}
    Error! set set `[tox] testenv-commands` in .meta.toml
{% endif %}
{% for line in testenv_additional %}
%(line)s
{% endfor %}

[testenv:setuptools-latest]
basepython = python3
deps =
    git+https://github.com/pypa/setuptools.git\#egg=setuptools
    zc.buildout
    wheel
{% for line in testenv_deps %}
    %(line)s
{% endfor %}
{% include 'tox-lint.j2' %}
{% if with_docs %}

[testenv:docs]
basepython = python3
deps =
   -r docs/requirements.txt
commands_pre =
commands =
    sphinx-build -b html -d docs/_build/doctrees docs docs/_build/html
{% if with_sphinx_doctests %}
    sphinx-build -b doctest -d docs/_build/doctrees docs docs/_build/doctest
{% endif %}
{% endif %}
