    strategy:
      fail-fast: false
      matrix:
        python-version:
{% if with_pypy %}
          - "pypy-%(pypy_version)s"
{% endif %}
{% for (py_long_version, py_short_version) in supported_python_versions %}
          - "%(py_long_version)s"
{% endfor %}
{% if with_future_python %}
          - "%(future_python_version)s"
{% endif %}
{% if with_windows %}
        os: [ubuntu-latest, macos-latest, windows-latest]
{% else %}
        os: [ubuntu-latest, macos-latest]
{% endif %}
{% if with_pypy or gha_additional_exclude %}
        exclude:
{% endif %}
{% if with_pypy %}
          - os: macos-latest
            python-version: "pypy-%(pypy_version)s"
{% endif %}
{% for line in gha_additional_exclude %}
          %(line)s
{% endfor %}
