{% if have_md_files %}
include *.md
{% endif %}
include *.rst
include *.txt
include buildout.cfg
include tox.ini
include .pre-commit-config.yaml
{% if with_docs %}

recursive-include docs *.py
recursive-include docs *.rst
{% if have_docs_txt_files %}
recursive-include docs *.txt
{% endif %}
recursive-include docs Makefile
{% endif %}

{% if have_src_folder %}
recursive-include src *.py
{% endif %}
{% for line in manifest_additional_rules %}
%(line)s
{% endfor %}
